"""
This is a `fedmsg consumer`_ that subscribes to every topic on the message bus
it is connected to. It just places all messages into a RabbitMQ message queue.
"""

import logging
import fedmsg.consumers
import json
import pika


log = logging.getLogger("fedmsg-rabbitmq-serializer")

OPTS = pika.ConnectionParameters(
    heartbeat_interval=0,
    retry_delay=2,
)


class RabbitMQSerializerConsumer(fedmsg.consumers.FedmsgConsumer):
    """
    A `fedmsg consumer`_ that subscribes to all topics and re-publishes all
    messages to the ``workers`` exchange.

    Attributes:
        topic (str): The topics this consumer is subscribed to. Set to ``*``
            (all topics).
        config_key (str): The key to set to ``True`` in the fedmsg config to
            enable this consumer. The key is ``rabbitmq.serializer.enabled``.
    """
    topic = '*'
    config_key = 'rabbitmq.serializer.enabled'

    def __init__(self, *args, **kwargs):
        log.debug("RabbitMQ Serializer initializing")
        super(RabbitMQSerializerConsumer, self).__init__(*args, **kwargs)
        log.debug("RabbitMQ Serializer initialized")

    def consume(self, raw_msg):
        """
        This method is called when a message arrives on the fedmsg bus.

        Args:
            raw_msg (dict): The raw fedmsg deserialized to a Python dictionary.
        """
        connection = pika.BlockingConnection(OPTS)
        channel = connection.channel()
        channel.exchange_declare(exchange='workers')
        channel.queue_declare('workers', durable=True)
        channel.basic_publish(
            exchange='',
            routing_key='workers',
            body=json.dumps(raw_msg),
            properties=pika.BasicProperties(
                delivery_mode=2
            )
        )
        channel.close()
        connection.close()

    def stop(self):
        """
        Gracefully halt this fedmsg consumer.
        """
        log.info("Cleaning up RabbitMQ Serializer.")
        super(RabbitMQSerializerConsumer, self).stop()
